@@ -46,7 +46,7 @@
 		int idx; \
 		for (idx = 0; idx < 16; idx++) { \
 			(dst)->s6_addr[idx] = \
-			    (src)->s6_addr[idx] & (msk)->s6_addr[idx]; \
+			    (u_char) ((src)->s6_addr[idx] & (msk)->s6_addr[idx]); \
 		} \
 	} while (0)
 
@@ -307,6 +307,8 @@ hack_restrict(
 	register struct restrictlist6 *rl6 = NULL;
 	register struct restrictlist6 *rlprev6 = NULL;
 	int i, addr_cmp, mask_cmp;
+	memset(&addr6, 0, sizeof(struct in6_addr)); 
+	memset(&mask6, 0, sizeof(struct in6_addr)); 
 
 	if (resaddr->ss_family == AF_INET) {
 		/*
@@ -359,7 +361,7 @@ hack_restrict(
 		SET_IPV6_ADDR_MASK(&addr6,
 		    &GET_INADDR6(*resaddr), &mask6);
 		if (IN6_IS_ADDR_UNSPECIFIED(&addr6)) {
-			rlprev6 = 0;
+			rlprev6 = NULL;
 			rl6 = restrictlist6;
 		} else {
 			rlprev6 = restrictlist6;
@@ -438,8 +440,13 @@ hack_restrict(
 				rl->mask = mask;
 				rl->mflags = (u_short)mflags;
 
-				rl->next = rlprev->next;
-				rlprev->next = rl;
+				if (rlprev == NULL) {
+					rl->next = restrictlist;
+					restrictlist = rl;
+				} else {
+					rl->next = rlprev->next;
+					rlprev->next = rl;
+				}
 				restrictcount++;
 			}
 			if ((rl->flags ^ (u_short)flags) &
@@ -475,7 +482,11 @@ hack_restrict(
 			if (rl != 0
 			    && rl->addr != htonl(INADDR_ANY)
 			    && !(rl->mflags & RESM_INTERFACE)) {
-				rlprev->next = rl->next;
+				if (rlprev != NULL) {
+					rlprev->next = rl->next;
+				} else {
+					restrictlist = rl->next;
+				}
 				restrictcount--;
 				if (rl->flags & RES_LIMITED) {
 					res_limited_refcnt--;
@@ -525,8 +536,13 @@ hack_restrict(
 				rl6->addr6 = addr6;
 				rl6->mask6 = mask6;
 				rl6->mflags = (u_short)mflags;
-				rl6->next = rlprev6->next;
-				rlprev6->next = rl6;
+				if (rlprev6 != NULL) {
+					rl6->next = rlprev6->next;
+					rlprev6->next = rl6;
+				} else {
+					rl6->next = restrictlist6;
+					restrictlist6 = rl6;
+				}
 				restrictcount6++;
 			}
 			if ((rl6->flags ^ (u_short)flags) &
@@ -562,7 +578,11 @@ hack_restrict(
 			if (rl6 != 0 &&
 			    !IN6_IS_ADDR_UNSPECIFIED(&rl6->addr6)
 			    && !(rl6->mflags & RESM_INTERFACE)) {
-				rlprev6->next = rl6->next;
+				if (rlprev6 != NULL) {
+					rlprev6->next = rl6->next;
+				} else {
+					restrictlist6 = rl6->next;
+				}
 				restrictcount6--;
 				if (rl6->flags & RES_LIMITED) {
 					res_limited_refcnt6--;
